package ink.wgink.module.examine.controller.api.paper;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.examine.pojo.dtos.paper.PaperDTO;
import ink.wgink.module.examine.pojo.vos.paper.PaperVO;
import ink.wgink.module.examine.service.paper.IPaperService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: PaperController
 * @Description: 试卷
 * @Author: WenG
 * @Date: 2020-09-11 11:50
 * @Version: 1.0
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "试卷接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/paper")
public class PaperController extends DefaultBaseController {

    @Autowired
    private IPaperService paperService;

    @ApiOperation(value = "新增试卷", notes = "新增试卷接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult savePaper(@RequestBody PaperVO paperVO) throws Exception {
        if (paperVO.getPassScore() > paperVO.getTotalScore()) {
            throw new ParamsException("及格分数不能大于总分");
        }
        paperService.save(paperVO);
        return new SuccessResult();
    }
    @ApiOperation(value = "删除试卷(id列表)", notes = "删除试卷(id列表)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("remove/{ids}")
    public SuccessResult remove(@PathVariable("ids") String ids) throws RemoveException {
        paperService.remove(Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改试卷", notes = "修改试卷接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "paperId", value = "试卷ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("update/{paperId}")
    @CheckRequestBodyAnnotation
    public SuccessResult updatePaper(@PathVariable("paperId") String paperId, @RequestBody PaperVO paperVO) throws Exception {
        if (paperVO.getPassScore() > paperVO.getTotalScore()) {
            throw new ParamsException("及格分数不能大于总分");
        }
        paperService.update(paperId, paperVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "试卷详情(通过ID)", notes = "试卷详情(通过ID)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "paperId", value = "试卷ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{paperId}")
    public PaperDTO getPaperById(@PathVariable("paperId") String paperId) throws SearchException {
        return paperService.getById(paperId);
    }

    @ApiOperation(value = "试卷列表", notes = "试卷列表接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<PaperDTO> list() throws SearchException {
        Map<String, Object> params = requestParams();
        return paperService.list(params);
    }

    @ApiOperation(value = "试卷分页列表", notes = "试卷分页列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<PaperDTO>> listPage(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return paperService.listPage(page);
    }

    @ApiOperation(value = "试卷统计", notes = "试卷统计接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("count")
    SuccessResultData<Integer> countPaper() throws SearchException {
        Map<String, Object> params = requestParams();
        Integer count = paperService.count(params);
        return new SuccessResultData<>(count);
    }

}